// This is raw javascript on purpose.
window.onload = function() {
    // I would normally recommend using jquery ready() but since I
    // want to keep this example as simple as possible, jquery is
    // assumed to not be available.


    // ================================================================
    // Test a condition and report a message.
    // ================================================================
    function assert(condition, description)
    {
        if(!condition) {
            alert('ASSERTION: ' + description + ': ' + condition);
        }
    }
    
    // ================================================================
    // Commaize a number.
    //    Insert thousands separator into a number and, optionally, specify
    //    the number of decimal places.
    //    
    //    Here are some examples that demonstrate how it is used.
    //    
    //      x=commaize(1234)                        # 1,234
    //      x=commaize(-1234)                       # -1,234
    //      x=commaize(12345.678)                   # 12,345.678
    //      x=commaize(-12345.678)                  # -12,345.678
    //      x=commaize(12345.678, 2)                # 12,345.69
    //      x=commaize(12345.678, 1)                # 12,345.7
    //      x=commaize(1234, -1, '.')               # 1.234
    //      x=commaize("1234567,89", -1, '.', ',')  # EU: 1.234.567,89
    //    
    //    The arguments are:
    //    
    //    @param num  The number: int, float or string.
    //    @param dpl  Decimal places (default: -1 -> do nothing).
    //    @param sep  Thousands separator (default: ",").
    //    @param dpt  Decimal point (default: ".").
    //    @returns a formatted string.
    // ================================================================
    function commaize(num, dpl=-1, sep=',', dpt='.')
    {
        var snum = num.toString();
        assert(snum.indexOf(sep) < 0, 'found "' + sep + '" in "' + snum + '"');

        // Format to N decimal places.
        if (dpl > 0) {
            snum = parseFloat(snum).toFixed(dpl).toString();
        }

        // Split into parts.
        var parts = snum.split(dpt);  // 123 or 123.45
        assert( parts.length < 3, 'too many parts: ' + parts.length);

        // Get the characteristic and mantissa.
        //   123.45
        //   ^  ^^
        //   |  |+--- mantissa
        //   |  +---- decimal point
        //   +------- characteristic
        var characteristic = parts[0];
        var mantissa = parts.length == 1 ? '' : dpt + parts[1];

        // Handle negative numbers.
        var neg = '';
        if (characteristic[0] == '-') {
            neg = '-';
            characteristic = characteristic.slice(1);  // strip off the negative
        }

        // Clean up the mantissa. Remove trailing zeros unless a specific
        // number of decimal places (dpl) was specified.
        //    1.000 > 1
        if (mantissa.length && dpl < 0) {
            // Make sure that 1.0 --> 1
            while (mantissa.slice(-1) == '0') {
                mantissa = mantissa.slice(0, -1);
            }
            if (mantissa == dpt) {
                mantissa = "";
            }
        }

        // Insert the commas moving from left to right.
        var result = '';  // where to put the result
        var offset = characteristic.length % 3  // offset to start inserting commas
        for(var i=0; i<characteristic.length; i++) {
            if ((i%3) == offset && i) {
                // if we are at the digit where a separator needs to be
                // pre-pended, prepend the separator.
                result += sep ;
            }
            result += characteristic[i];
        }

        return neg + result + mantissa;
    }

    // ================================================================
    // main
    // ================================================================
    function main()
    {
        // This function will generate a table on the page with the
        // filled in contents of the commaize function for the examples
        // that we are interested in.
        var rowidx = 0;
        row = function(val, result, msg='')
        {
            var s1 = "" + rowidx;
            var s2 = "" + val;
            var s3 = "" + result;
            var html = '';

            rowidx++;

            // Pad.
            while (s1.length < 4) {s1 = ' ' + s1;}
            while (s2.length < 20) {s2 = ' ' + s2;}
            while (s3.length < 20) {s3 = ' ' + s3;}

            html += s1 + ' ' + s2 + '  ' + s3;
            if (msg.length > 0) {
                html += '  ' + msg;
            }
            html += '\n';
            
            return html;
        }

        var place = document.getElementById('place');
        var html = '';
        html += '<pre>\n';
        html += 'BEGIN\n';

        // Rows of data here.
        var data = [1, 12, 123, 1234, 12345, 123456, 1234567, 12345678, 123456789,
                    1.23, 12.34, 123.45, 1234.56, 12345.67, 123456.78, 1234567.89, 12345678.9,
                    1.0,
                    -1, -12, -123, -1234, -12345, -123456, -1234567, -12345678, -123456789,
                    -1.23, -12.34, -123.45, -1234.56, -12345.67, -123456.78, -1234567.89, -12345678.9,
                    -1.0,
                    '123456789',
                   ];
        for(var i=0; i<data.length; i++) {
            var datum = data[i];
            var result = commaize(datum);
            html += row(datum, result);
        }

        // EU style
        var datum = "12345678,9";
        var result = commaize(datum, -1, '.', ',');  // EU style
        html += row(datum, result, '# EU style');

        // dollarize
        // Round to 2 significant digits.
        var data1 = [12345.123, 12345, 12345.1251,];
        for(var i=0; i<data1.length; i++) {
            datum = data1[i];
            result = commaize(datum, 2);
            html += row(datum, result, '# 2 decimal places');
        }

        // 1 decimal place
        datum = "1234567.89";
        result = commaize(datum, 1)
        html += row(datum, result, '# 1 decimal place');

        html += 'END\n';
        html += '</pre>\n';
        place.innerHTML = html;
    }

    main();
}
